# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class MetadataRegexValidationRule(UniversalBaseModel):
    regex: str = pydantic.Field()
    """
    A regular expression that the value must match.
    """

    error_message: str = pydantic.Field(alias="errorMessage")
    """
    The error message to display if the value does not match the regular expression.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
