# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceDateFilter = typing.Union[
    typing.Literal[
        "INVOICE_DATE", "DEDUCTION_DATE", "NEXT_PAYMENT_DATE", "DUE_DATE", "SETTLEMENT_DATE", "CREATED_AT", "UPDATED_AT"
    ],
    typing.Any,
]
