# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class Ein(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import Ein

    Ein(
        number="12-3456789",
    )
    """

    number: str = pydantic.Field()
    """
    Must be in the format XX-XXXXXXX.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
