# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from ...invoice_types.types.invoice_id import InvoiceId
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class InvoiceTiming(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.calculate import InvoiceTiming

    InvoiceTiming(
        invoice_id="inv_26e7b5d3-a739-4b23-9ad9-6aaa085f47a9",
    )
    """

    invoice_id: InvoiceId = pydantic.Field(alias="invoiceId")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
