# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceOrderByField = typing.Union[
    typing.Literal[
        "AMOUNT",
        "DUE_DATE",
        "CREATED_AT",
        "UPDATED_AT",
        "DEDUCTION_DATE",
        "INVOICE_DATE",
        "SETTLEMENT_DATE",
        "INVOICE_NUMBER",
        "VENDOR_NAME",
        "PAYER_NAME",
        "NEXT_PAYMENT_DATE",
    ],
    typing.Any,
]
