# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class IndividualGovernmentId(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.commons import IndividualGovernmentId

    IndividualGovernmentId(
        ssn="123-45-6789",
    )
    """

    ssn: str = pydantic.Field()
    """
    Full Social Security Number. Must be in the format 123-45-6789.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
