# This file was auto-generated by Fern from our API Definition.

import typing
from .estimated_timing import EstimatedTiming
from .invoice_timing import InvoiceTiming

CalculatePaymentTimingRequest = typing.Union[EstimatedTiming, InvoiceTiming]
