# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .payment_methods_request import PaymentMethodsRequest
from .email_provider_request import EmailProviderRequest
from .external_accounting_system_provider_request import ExternalAccountingSystemProviderRequest
from .color_scheme_request import ColorSchemeRequest
from .onboarding_options_request import OnboardingOptionsRequest
from .metadata_schema import MetadataSchema
from .notification_email_template_request import NotificationEmailTemplateRequest
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class OrganizationRequest(UniversalBaseModel):
    name: typing.Optional[str] = None
    logo: typing.Optional[str] = None
    website_url: typing.Optional[str] = pydantic.Field(alias="websiteUrl", default=None)
    support_email: typing.Optional[str] = pydantic.Field(alias="supportEmail", default=None)
    payment_methods: typing.Optional[PaymentMethodsRequest] = pydantic.Field(alias="paymentMethods", default=None)
    email_provider: typing.Optional[EmailProviderRequest] = pydantic.Field(alias="emailProvider", default=None)
    external_accounting_system_provider: typing.Optional[ExternalAccountingSystemProviderRequest] = pydantic.Field(
        alias="externalAccountingSystemProvider", default=None
    )
    color_scheme: typing.Optional[ColorSchemeRequest] = pydantic.Field(alias="colorScheme", default=None)
    payee_onboarding_options: typing.Optional[OnboardingOptionsRequest] = pydantic.Field(
        alias="payeeOnboardingOptions", default=None
    )
    payor_onboarding_options: typing.Optional[OnboardingOptionsRequest] = pydantic.Field(
        alias="payorOnboardingOptions", default=None
    )
    metadata_schema: typing.Optional[typing.List[MetadataSchema]] = pydantic.Field(alias="metadataSchema", default=None)
    notification_email_template: typing.Optional[NotificationEmailTemplateRequest] = pydantic.Field(
        alias="notificationEmailTemplate", default=None
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
